﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.IO;
using Examine.LuceneEngine;
using Examine;

namespace ExamineDemo
{

    /// <summary>
    /// The data service used by the LuceneEngine in order for it to reindex all data
    /// </summary>
    public class CustomDataService : Examine.LuceneEngine.ISimpleDataService
    {
        /// <summary>
        /// loads the data source into memory
        /// </summary>
        static CustomDataService()
        {
            DataSource = XDocument.Load(Path.Combine(Environment.CurrentDirectory, "SampleData.xml"));
        }

        private static XDocument DataSource;

        #region ISimpleDataService Members

        /// <summary>
        /// Returns a list of type SimpleDataSet based on the SampleData.xml data
        /// </summary>
        /// <param name="indexType"></param>
        /// <returns></returns>
        public IEnumerable<Examine.LuceneEngine.SimpleDataSet> GetAllData(string indexType)
        {
            var data = new List<SimpleDataSet>();

            //open the XML file and iterate the nodes
            foreach (var item in DataSource.Root.Elements())
            {
                //add a new SimpleDataSet object to the list

                data.Add(new SimpleDataSet()
                {
                    //create the node definition, ensure that it is the same type as referenced in the config
                    NodeDefinition = new IndexedNode()
                    {
                        NodeId = (int)item.Attribute("id"),
                        Type = "CustomData"
                    },
                    //add the data to the row
                    RowData = new Dictionary<string, string>() 
                    {
                         {"id", (string)item.Attribute("id")},
                         {"name", (string)item.Attribute("name")},
                         {"description", (string)item.Attribute("description")},
                         {"dateUpdated", (string)item.Attribute("dateUpdated")}
                     }
                });
            }

            return data;
        }

        #endregion

        /// <summary>
        /// Update the timestamp for the node with the specified id
        /// </summary>
        /// <param name="nodeId"></param>
        /// <returns>Returns a dictionary of the data structure</returns>
        public static Dictionary<string, string> UpdateTimeStamp(int nodeId)
        {            
            var node = DataSource.Root.Elements().Where(x => (int)x.Attribute("id") == nodeId).SingleOrDefault();
            if (node == null)
                throw new KeyNotFoundException("No node with id " + nodeId + " found");

            node.Attribute("id").Value = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");

            //return a dictionary object of the node

            return new Dictionary<string, string>() 
                        {
                            {"id", (string)node.Attribute("id")},
                            {"name", (string)node.Attribute("name")},
                            {"description", (string)node.Attribute("description")},
                            {"dateUpdated", (string)node.Attribute("dateUpdated")}
                        };
        }
    }
}
