﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Examine;
using Examine.SearchCriteria;
using Examine.LuceneEngine;

namespace ExamineDemo
{
    class Program
    {
        /// <summary>
        /// Does some stuff
        /// </summary>
        /// <remarks>
        /// 1. This re-indexes all data
        /// 2. Searches the index for node id 1
        /// 3. Ensures one record is found in the index
        /// 4. Updates the dateUpdated time stamp for the data record
        /// 5. Re-indexes the record with node id 1
        /// </remarks>
        /// <param name="args"></param>
        static void Main(string[] args)
        {

            //first, let's wire up some events for tracking just to display on the console
            ExamineManager.Instance.IndexProviderCollection[0].NodeIndexing += new EventHandler<IndexingNodeEventArgs>(Program_NodeIndexing);
            ExamineManager.Instance.IndexProviderCollection[0].NodeIndexed += new EventHandler<IndexedNodeEventArgs>(Program_NodeIndexed);
            ExamineManager.Instance.IndexProviderCollection[0].NodesIndexed += new EventHandler<IndexedNodesEventArgs>(Program_NodesIndexed);

            //rebuild the whole index
            Console.WriteLine("Re-indexing all data...");
            ExamineManager.Instance.RebuildIndex();

            //do some searching, updating and re-indexing just for fun

            //search for node id 1
            var criteria = ExamineManager.Instance.CreateSearchCriteria().Id(1).Compile();
            var result1 = ExamineManager.Instance.Search(criteria);
            Console.WriteLine("Found " + result1.Count() + " result for query: " + criteria.ToString());

            //ensure there's only one result 
            if (result1.Count() > 1)
                throw new InvalidOperationException("Search result must only return one record");

            //update the record
            var dataSet = CustomDataService.UpdateTimeStamp(result1.First().Id);

            //now, re-index the node manually...
            //Examine is based on the XML structure that Umbraco runs on which should in theory support all kinds of data,
            //so we use a nice little extension method for a Dictionary<string, string> to convert to the Examine XML structure
            ExamineManager.Instance.ReIndexNode(dataSet.ToExamineXml(result1.First().Id, "CustomData"), "CustomData");

            Console.WriteLine("Press any key to exit");
            Console.Read();
        }

        #region Event handlers
        static void Program_NodeIndexing(object sender, IndexingNodeEventArgs e)
        {
            Console.WriteLine("Indexing node " + e.NodeId + "...");
        }

        static void Program_NodeIndexed(object sender, IndexedNodeEventArgs e)
        {
            Console.WriteLine("... DONE");
        }

        static void Program_NodesIndexed(object sender, IndexedNodesEventArgs e)
        {
            Console.WriteLine(e.Nodes.Count() + " nodes indexed");
        } 
        #endregion
        
    }
}
